/**
 * @license
 * Visual Blocks Language
 *
 * Copyright 2012 Google Inc.
 * https://developers.google.com/blockly/
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

/**
 * @fileoverview Generating JavaScript for text blocks.
 * @author vwinter@unomaha.edu (Victor Winter)
 */
'use strict';

goog.provide('Blockly.JavaScript.texts');

goog.require('Blockly.JavaScript');

// ===================================================================================
// ----------------------------------- Build -----------------------------------------
// ===================================================================================
Blockly.JavaScript['bbuild'] = function(block) {
 
  var size = block.getFieldValue('size');
  var sym  = Blockly.JavaScript.valueToCode(block, 'semi_or_epsilon', Blockly.JavaScript.ORDER_ADDITION) || '0';
 
  processBegin('build','build',sym);
  
  dimensions = 1;
 
  if (size == "(2,2)"    ) { dimensions =   2; } 
  if (size == "(4,4)"    ) { dimensions =   4; }
  if (size == "(8,8)"    ) { dimensions =   8; }  
  if (size == "(16,16)"  ) { dimensions =  16; }    
  if (size == "(32,32)"  ) { dimensions =  32; } 
  if (size == "(64,64)"  ) { dimensions =  64; } 
  if (size == "(128,128)") { dimensions = 128; }
  
  //initGame(dimensions);

  var code = 'initGame(' + dimensions + ');\n';
  
  processEnd(NO_TRANSITION,"build2D " + size + sym + '\n'); // want extra newline 
  
  return code;
};

// ===================================================================================
// ----------------------------------- Show ------------------------------------------
// ===================================================================================
Blockly.JavaScript['show'] = function(block) {
 
  var artifactName = block.getFieldValue('artifact');
  var sym          = Blockly.JavaScript.valueToCode(block, 'semi_or_epsilon', Blockly.JavaScript.ORDER_ADDITION) || '1'; 
  var code         = 'show2D(' + artifactName + ');\n';
 
  processBegin('show','show2D',sym); 
  processEnd(SHOW,'show2D ' + artifactName + sym);
  
  return code;
};


Blockly.JavaScript['bcomment'] = function(block) {
 
  var msg   = block.getFieldValue('msg');
  var code  = ' ';
  
  processEnd(COMMENT,'(* ' + msg + ' *)');
   
  return code;
};


