/* The purpose of this state machine is to track when
   semicolons are necessary. They are necessary in most
   cases. However, a semicolon may not be the last symbol
   in an expression sequence.
*/   

var NO_TRANSITION = -1; 
// FA states
var START   = 0;
var DECS    = 1;
var IN_FUN  = 2;
var CREATE  = 3;
var DISPLAY = 4;  // accept state
var REJECT  = 5;  // reject state

// transition symbols
var OPEN      = 0;
var DEC_BEGIN = 1;
var DEC_END   = 2;
var CALL      = 3;
var BUILD     = 4;
var SHOW      = 5;
var COMMENT   = 6;

var STATES      = 6;
var TRANSITIONS = 6;

var trace = false;

var FA_currentState = START;
var TRANSITION_TABLE; // A[state][symbol] = state

// -----------------------------------------------------------------------------------
var ERROR = false;


// this only accounts for the most basic cases. Missing cases include:
// 
//   myFun(5 + 07,6+09);
//   incOffset2D(5+07,6+09);
//   etc.
//
function wellFormedInt(v) {
    if (v.charAt(0) == "0" && v.length > 1) {
        ERROR = true;
        handleException("\n\tOctal Error: A base 10 integer may not have leading zeros.");
    }
}

// http://www.w3schools.com/jsref/jsref_obj_regexp.asp
// test()	Tests for a match in a string. Returns true or false
// alphaNumeric = ( [a-z] | [A-Z] | [0-9] )
// alpha        = ( [a-z] | [A-Z] )
//
// Quantifiers:
// n+	Matches any string that contains at least one n
//

// Press the F12 function key in the Chrome browser to launch the JavaScript debugger and then click "Scripts".
function wellFormedFunName(str) {
    
    var alpha             = /[A-z]/;
    var first             = str.charAt(0);
    var matchesFirstAlpha = first.match(alpha); // will match underscore
    
    //alert('first = ' + str.charAt(0) + ' matchesFirstAlpha = ' + matchesFirstAlpha);
    
    var notAlphanumeric        = /([^A-z,0-9])/;
    var matchesNotAlphanumeric = str.match(notAlphanumeric);

    //alert(' matchesNotAlphanumeric = ' + matchesNotAlphanumeric);    
    
    if (!matchesFirstAlpha || matchesNotAlphanumeric) {
        ERROR = true;
        handleException("\n\tThe string: \"" + str + "\" is not a legal function name.");
    }
}

// -----------------------------------------------------------------------------------
function stateToStr(someState) {
    var result;
    
    switch (someState) {
        case 0: result = "start";
                break;
        case 1: result = "decs";
                break;
        case 2: result = "in_fun";
                break;
        case 3: result = "create";
                break;
        case 4: result = "display";
                break;
        case 5: result = "reject";
                break;
                               
    }
    return result;
}

// -----------------------------------------------------------------------------------
function initStateMachine() {
    
    FA_currentState = START;
    
    TRANSITION_TABLE = new Array(STATES);
    for (i = 0; i < STATES; i++) {
        TRANSITION_TABLE[i] = new Array(TRANSITIONS);
    }
      
    for (i = 0; i < STATES; i++) {
        for (j = 0; j < TRANSITIONS; j++) {
            TRANSITION_TABLE[i][j] = REJECT;
        }    
    }
    
    TRANSITION_TABLE[START][OPEN] = DECS;
    
    TRANSITION_TABLE[DECS][DEC_BEGIN] = IN_FUN; 
    TRANSITION_TABLE[DECS][BUILD]     = CREATE;     
    
    TRANSITION_TABLE[IN_FUN][CALL]    = IN_FUN;  
    TRANSITION_TABLE[IN_FUN][DEC_END] = DECS;  
    
    TRANSITION_TABLE[CREATE][CALL]    = CREATE;  
    TRANSITION_TABLE[CREATE][SHOW]    = DISPLAY;  
}

// -----------------------------------------------------------------------------------
function resetFA() {
    FA_currentState = START;
}

// -----------------------------------------------------------------------------------
function transitionFA(symbol) {
    if (symbol == NO_TRANSITION) {
            // do nothing
    }
    else {
            var nextState = TRANSITION_TABLE[FA_currentState][symbol];
            var msg = "TRANSITION_TABLE[ " + FA_currentState + "][" + symbol + "] = " 
                      + stateToStr(nextState);
                      
            FA_currentState = nextState;
            
            if (trace) {
                alert(msg);
            }
            if (FA_currentState == REJECT) {
               alert("FA reject state");
            }
    }
}

// ===================================================================================
// ----------------------------------- program state ----------------------------------
// ===================================================================================

var version        = "1.2";
var versionComment = "(*& ====================================================\n"  
                      +  "    @source   bricklayerLite version " + version + "\n"
                      +  "    @author   your name\n" 
                      +  "    @class    class-id\n" 
                      +  "    @exercise exercise-id\n" 
                      +  "    @year     2016\n" 
                      +  "    ====================================================\n &*)\n\n";

// --------------------------------
// symbols
// --------------------------------
var lparen    = "(";
var rparen    = ")";
var empty     = " ";
var semicolon = ";";
var initialLastSymbol = "dummy";

// --------------------------------
// Bricklayer state
// --------------------------------
var open_Level_1              = false;
var open_Level_2              = false;
var open_Level_3              = false;
var build                     = false;
var showState                 = false;
var bricklayerCode            = versionComment;
var codeBuffer                = '';
var lastSymbol                = initialLastSymbol;
var blockCount                = 0;
var blockStr                  = '';
var inScope                   = false;
var LM                        = 0; // number of tabs
var xOffset                   = 0;
var zOffset                   = 0;

function initBricklayerState () {
    open_Level_1              = false;
    open_Level_2              = false;
    open_Level_3              = false;
    build                     = false;
    showState                 = false;
    bricklayerCode            = versionComment;
    codeBuffer                = '';
    lastSymbol                = initialLastSymbol;
    blockCount                = 0;
    blockStr                  = ''; 
    inScope                   = false;   
    LM                        = 0; 
    xOffset                   = 0;    
    zOffset                   = 0;
}

// ---------------------------------------------------------------------------------------
function indent() {
    var str = '';
    
    for (i = 0; i < LM; i++) {
        str = str + '\t';
    }
   
    return str;
}
// ---------------------------------------------------------------------------------------
function funBodyWellformed(body) {
    var wellformed = false;
    
    if (lastSymbol == semicolon) {
      alert("error -- expr-seq may not have trailing semicolon");
    }  
    else {
        if (body == 1) {
            alert("malformed function declaration");         
        } 
        else { wellformed = true; }
    }
    
    return wellformed;
}

// ---------------------------------------------------------------------------------------    
function inShowState () {
    return showState;
}    

function inPutState () {
    return (open_Level_1 || open_Level_2 || open_Level_3 ) && build;
}    

function inappropriateStateForPut() {
    var msg = "\nError: You cannot issue a put function call at this point.\n";
    
    if (!(open_Level_1 || open_Level_2 || open_Level_3)) {
        msg = msg + "You must open a level.\n";
    }
    
    if (!(build)) {
        msg = msg + "You must build a base plate.\n";
    }
    
    bricklayerCode = bricklayerCode + msg + "\n";
}    

// ---------------------------------------------------------------------------------------    
function addToBricklayerCode(str) {  
    if (inScope) {
        codeBuffer = codeBuffer + str;  
    }
    else {
        bricklayerCode = bricklayerCode + str;  
    }
}    

// ---------------------------------------------------------------------------------------    
function showBricklayerCode() { addContent("bricklayerCode",bricklayerCode); }

// ===================================================================================
// ----------------------------------- analysis --------------------------------------
// ===================================================================================
// -----------------------------------------------------------------------------------
function inFun() {
    return FA_currentState == IN_FUN;
}

// -----------------------------------------------------------------------------------
function symAcceptable( sym ) {
    var flag = inFun() || sym == semicolon;
       
    if (sym == 1) {
         throw("missing symbol");
    }     
    if (!flag) {
        throw("ill formed");
    }
}

// -----------------------------------------------------------------------------------
function decAcceptable() {
   
    if (inFun()) {
        throw("Error: a function may not be declared within the body of a function (i.e., within expr seq).");
    }
}

// -----------------------------------------------------------------------------------
function checkLastSymbol() {
   
    if (lastSymbol == ';') {
        throw("\n\n***Error: The last symbol in the body of a function declaration may not be a semi-colon.");
    }
}

// -----------------------------------------------------------------------------------
function assertLevel2or3(name) {
   
    if (!open_Level_2 && !open_Level_3) {
        throw('You must open Level_2 or Level_3 to access the function ' + name);
    }
}

// -----------------------------------------------------------------------------------
function assertLevel3(name) {
   
    if (!open_Level_3) {
        throw('You must open Level_3 to access the function ' + name);
    }
}

// -----------------------------------------------------------------------------------
function processBegin(str,info,sym) {
    var transitionSymbol;
    
    blockCount++;     // error message tracking
    blockStr = info;  // error message info
    
    symAcceptable(sym);
    
    if (str == 'open') {        
        transitionSymbol = OPEN;
    }

    if (str == 'declaration_nullaryFunction') {
        transitionSymbol = DEC_BEGIN;
        LM = LM + 2;
        inScope = true;
    }

    if (str == 'declaration_function') {
        transitionSymbol = DEC_BEGIN;
        LM = LM + 2;
        inScope = true;
    }
    
    if (str == 'build') {
        transitionSymbol = BUILD;
        build = true;
    }

    if (str == 'show') {
        transitionSymbol = NO_TRANSITION;
    }

    if (str == 'call') {
        transitionSymbol = CALL;
        lastSymbol = sym;
    }

    if (str == 'circleXZ') {
        
        assertLevel2or3(str);
        
        transitionSymbol = CALL;
        lastSymbol = sym;
    }    

    if (str == 'ringXZ') {
        
        assertLevel2or3(str);
        
        transitionSymbol = CALL;
        lastSymbol = sym;
    }    
    
    if (str == 'offset') {
        
        assertLevel2or3(str);
        
        transitionSymbol = CALL;
        lastSymbol = sym;
    }    

    if (str == 'put2D') {
        
        assertLevel3(str);
        
        transitionSymbol = CALL;
        lastSymbol = sym;
    }    

    if (str == 'lineXZ') {
        
        assertLevel3(str);
        
        transitionSymbol = CALL;
        lastSymbol = sym;
    }    
    
    transitionFA(transitionSymbol);    
}

// -----------------------------------------------------------------------------------
function processEnd(transitionSymbol,sourceText) {
   var defaultFormat = '\n' + indent();
   
   switch (transitionSymbol) {
       case -1: // NO_TRANSITION
       {
           addToBricklayerCode(defaultFormat + sourceText);
           break;
       }
       case 2: // DEC_END 
       {
           var sourceCode;
           var parenLM;
           
           LM--;       
           parenLM = indent();           
           LM--;
           
           sourceCode = '\n' + indent() + 'fun ' + sourceText + '=' 
                        + '\n' + parenLM + '(' + codeBuffer + '\n' + parenLM + ');\n';
           bricklayerCode = bricklayerCode + sourceCode;
           codeBuffer = '';
           inScope = false;           
           
           checkLastSymbol();
           
           break;
       }
       case 3: // CALL
       {
           addToBricklayerCode(defaultFormat + sourceText);
           break;
       }
       
       case 5: // SHOW
       {    
           addToBricklayerCode('\n' + defaultFormat + sourceText);           
           showState = true;
           break;
       }

       case 6: // COMMENT
       {
            // may want to add extra newline in some cases          
           addToBricklayerCode('\n' + defaultFormat + sourceText); 
           transitionSymbol = NO_TRANSITION;                    
           break;
       }
       
       default: {
           throw 'error in processScopeExit';
       }
   }
  
    transitionFA(transitionSymbol);
    
}

// -----------------------------------------------------------------------------------

function handleException(msg) {
    var errorMessage = msg + "\n\tblock number = " + blockCount + "\n\tblock = " + blockStr;
    
    bricklayerCode = bricklayerCode + errorMessage;
    
    showBricklayerCode();
}
