/**
 * @license
 * Visual Blocks Language
 *
 * Copyright 2012 Google Inc.
 * https://developers.google.com/blockly/
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

/**
 * @fileoverview Generating JavaScript for text blocks.
 * @author vwinter@unomaha.edu (Victor Winter)
 */
'use strict';

goog.provide('Blockly.JavaScript.texts');

goog.require('Blockly.JavaScript');

// ===================================================================================
// ----------------------------------- Declarations ----------------------------------
// ===================================================================================
Blockly.JavaScript['bopen'] = function(block) {
 
  var which_level = block.getFieldValue('level');
  var sym  = Blockly.JavaScript.valueToCode(block, 'semi_or_epsilon', Blockly.JavaScript.ORDER_ADDITION) || '0';

  processBegin('open','open',sym);
  
  if (which_level == "Level_1"  ) { open_Level_1 =  true; }
  if (which_level == "Level_2"  ) { open_Level_2 =  true; }  
  if (which_level == "Level_3"  ) { open_Level_3 =  true; }  
  
  var code = ' ';
 
  processEnd(NO_TRANSITION,'open ' + which_level + sym + '\n');
  
  return code;
};

// -----------------------------------------------------------------------------------
Blockly.JavaScript['declaration_nullaryFunction'] = function(block) { 
 
  var sym      = Blockly.JavaScript.valueToCode(block, 'semi_or_epsilon', Blockly.JavaScript.ORDER_ADDITION) || '0';  
  processBegin('declaration_nullaryFunction','nullary function declaration',sym);
  
  var nameOfFn = block.getFieldValue('nameOfFn');
  var body     = Blockly.JavaScript.statementToCode(block, 'body', Blockly.JavaScript.ORDER_ADDITION) || '1'; 
  var code     = 'function ' + nameOfFn + ' () ' + '{ ' + body + ' }\n';
    
  processEnd(DEC_END, nameOfFn + ' () ');
  return code;
};

// -----------------------------------------------------------------------------------
Blockly.JavaScript['declaration_function'] = function(block) { 
 
  var sym      = Blockly.JavaScript.valueToCode(block, 'semi_or_epsilon', Blockly.JavaScript.ORDER_ADDITION) || '0';  
  processBegin('declaration_function','parameterized function declaration',sym);
  
  var nameOfFn = block.getFieldValue('nameOfFn');
  var xParam   = block.getFieldValue('x');
  var zParam   = block.getFieldValue('z');
  var body     = Blockly.JavaScript.statementToCode(block, 'body', Blockly.JavaScript.ORDER_ADDITION) || '1'; 
  var code     = 'function ' + nameOfFn + ' (' + xParam + ',' + zParam + ') ' + '{ ' + body + ' }\n';
    
  processEnd(DEC_END, nameOfFn + ' (' + xParam + ',' + zParam + ') ');
  return code;
};

// ===================================================================================
// ----------------------------------- Build -----------------------------------------
// ===================================================================================
Blockly.JavaScript['bbuild'] = function(block) {
    
  var size = block.getFieldValue('size');
  var sym  = Blockly.JavaScript.valueToCode(block, 'semi_or_epsilon', Blockly.JavaScript.ORDER_ADDITION) || '0';
  
  processBegin('build','build',sym);
  
  dimensions = 1;
 
  if (size == "(2,2)"    ) { dimensions =   2; } 
  if (size == "(4,4)"    ) { dimensions =   4; }
  if (size == "(8,8)"    ) { dimensions =   8; }  
  if (size == "(16,16)"  ) { dimensions =  16; }    
  if (size == "(32,32)"  ) { dimensions =  32; } 
  if (size == "(64,64)"  ) { dimensions =  64; } 
  if (size == "(128,128)") { dimensions = 128; }
  
  //initGame(dimensions);

  var code = 'initGame(' + dimensions + ');\n';
  
  processEnd(NO_TRANSITION,"build2D " + size + sym + '\n'); // want extra newline 
  return code;
};


// ===================================================================================
// ----------------------------------- function calls --------------------------------
// ===================================================================================
  
Blockly.JavaScript['bcallNullary'] = function(block) { 
  
  var nameOfFn = block.getFieldValue('userFunName');
  var sym      = Blockly.JavaScript.valueToCode(block, 'semi_or_epsilon', Blockly.JavaScript.ORDER_ADDITION) || '1'; 
  var code     = nameOfFn + '();\n';
 
  processBegin('call','call',sym); 
  processEnd(CALL,nameOfFn + '()' + sym);
 
  return code;
};

  
Blockly.JavaScript['bcall'] = function(block) { 
  
  var nameOfFn = block.getFieldValue('userFunName');
  var xArg     = block.getFieldValue('xArg');
  var zArg     = block.getFieldValue('zArg');
  var sym      = Blockly.JavaScript.valueToCode(block, 'semi_or_epsilon', Blockly.JavaScript.ORDER_ADDITION) || '1'; 
  var code     = nameOfFn + '(' + xArg + ',' + zArg + ');\n';
 
  processBegin('call','call',sym); 
  processEnd(CALL,nameOfFn +  '(' + xArg + ',' + zArg + ')' + sym);
 
  return code;
};
 
// ===================================================================================
// ----------------------------------- Show ------------------------------------------
// ===================================================================================
Blockly.JavaScript['show'] = function(block) {
 
  var artifactName = block.getFieldValue('artifact');
  var sym          = Blockly.JavaScript.valueToCode(block, 'semi_or_epsilon', Blockly.JavaScript.ORDER_ADDITION) || '1'; 
  var code         = 'show2D(' + artifactName + ');\n';
 
  processBegin('show','show2D',sym); 
  processEnd(SHOW,'show2D ' + artifactName + sym);
  
  return code;
};


Blockly.JavaScript['bcomment'] = function(block) {
 
  var msg   = block.getFieldValue('msg');
  var code  = ' ';
  
  processEnd(COMMENT,'(* ' + msg + ' *)');
   
  return code;
};


// ===================================================================================
// ------------------------------- Offsets -------------------------------------------
// ===================================================================================
Blockly.JavaScript['setOffset2D'] = function(block) {
    
    var name      = 'setOffset2D';
    var text_xval = block.getFieldValue('xVal');
    var text_zval = block.getFieldValue('zVal');
    var sym       = Blockly.JavaScript.valueToCode(block, 'semi_or_epsilon', Blockly.JavaScript.ORDER_ADDITION) || '1';     
    var code      = 'setOffset(' + text_xval + ',' + text_zval + ');\n';    

    processBegin('Level_2', name + '(' + text_xval + ',' + text_zval + ')',sym);         
    processEnd  (CALL  , name + '(' + text_xval + ',' + text_zval + ')' + sym);
    
    return code;
};

// -----------------------------------------------------------------------------------
Blockly.JavaScript['incOffset2D'] = function(block) {
    
    var name      = 'incOffset2D';
    var text_xval = block.getFieldValue('xVal');
    var text_zval = block.getFieldValue('zVal');
    var sym       = Blockly.JavaScript.valueToCode(block, 'semi_or_epsilon', Blockly.JavaScript.ORDER_ADDITION) || '1';     
    var code      = 'incOffset(' + text_xval + ',' + text_zval + ');\n';    

    processBegin('Level_2', name + '(' + text_xval + ',' + text_zval + ')',sym);         
    processEnd  (CALL  , name + '(' + text_xval + ',' + text_zval + ')' + sym);
    
    return code;
};

// -----------------------------------------------------------------------------------
Blockly.JavaScript['decOffset2D'] = function(block) {
    
    var name      = 'decOffset2D';
    var text_xval = block.getFieldValue('xVal');
    var text_zval = block.getFieldValue('zVal');
    var sym       = Blockly.JavaScript.valueToCode(block, 'semi_or_epsilon', Blockly.JavaScript.ORDER_ADDITION) || '1';     
    var code      = 'decOffset(' + text_xval + ',' + text_zval + ');\n';    

    processBegin('Level_2', name + '(' + text_xval + ',' + text_zval + ')',sym);         
    processEnd  (CALL  , name + '(' + text_xval + ',' + text_zval + ')' + sym);
    
    return code;
};

// ===================================================================================
// ----------------------------------- Shapes ----------------------------------------
// ===================================================================================
Blockly.JavaScript['circleXZ'] = function(block) {
    
    var name   = 'circleXZ';
    var radius = block.getFieldValue('radius');
    var brick  = block.getFieldValue('brick');    
    var x      = block.getFieldValue('xArg');
    var z      = block.getFieldValue('zArg');
    var sym    = Blockly.JavaScript.valueToCode(block, 'semi_or_epsilon', Blockly.JavaScript.ORDER_ADDITION) || '1';     
    var code   = name + '(' + radius + ',' + '\'' + brick + '\'' + ',' + x + ',' + z + ');\n';    

    var brickId = colorToBrick(brick);
    
    processBegin('circleXZ', name + ' ' + radius + ' ' + brickId + ' (' + x + ',' + z + ')',sym);         
    processEnd  (CALL  , name + ' ' + radius + ' ' + brickId + ' (' + x + ',' + z + ')' + sym);
       
    return code;    
}

Blockly.JavaScript['ringXZ'] = function(block) {
    
    var name      = 'ringXZ';
    var radius    = block.getFieldValue('radius');
    var thickness = block.getFieldValue('thickness');
    var brick     = block.getFieldValue('brick');    
    var x         = block.getFieldValue('xArg');
    var z         = block.getFieldValue('zArg');
    var sym       = Blockly.JavaScript.valueToCode(block, 'semi_or_epsilon', Blockly.JavaScript.ORDER_ADDITION) || '1';     
    var code      = name + '(' + radius + ',' + thickness + ',' + '\'' + brick + '\'' + ',' + x + ',' + z + ');\n';    

    var brickId = colorToBrick(brick);
    
    processBegin('ringXZ', name + ' ' + radius + ' ' + thickness + ' ' + brickId +  ' (' + x + ',' + z + ')',sym);         
    processEnd  (CALL    , name + ' ' + radius + ' ' + thickness + ' ' + brickId +  ' (' + x + ',' + z + ')' + sym);
       
    return code;    
}


// ===================================================================================
// ------------------------------- Symbols -------------------------------------------
// ===================================================================================
Blockly.JavaScript['bwhat'] = function(block) {
  var text_symbol = block.getFieldValue('sym');
  var code = ';';
 
  lastSymbol = semicolon;
  
  return code;
};

Blockly.JavaScript['bempty'] = function(block) {
  var code = ' ';
 
  lastSymbol = empty;
  
  return code;
};

Blockly.JavaScript['lparen'] = function(block) {
  var code = ' ';
  
  lastSymbol = lparen;
  
  return code;
};

Blockly.JavaScript['rparen'] = function(block) {
  var code = ' ';
  
  lastSymbol = rparen;
  
  return code;
};

// ===================================================================================
// ------------------------------- Experimental --------------------------------------
// ===================================================================================


