/**
 * @license
 * Visual Blocks Language
 *
 * Copyright 2012 Google Inc.
 * https://developers.google.com/blockly/
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

/**
 * @fileoverview Generating JavaScript for text blocks.
 * @author vwinter@unomaha.edu (Victor Winter)
 */
'use strict';

goog.provide('Blockly.JavaScript.texts');

goog.require('Blockly.JavaScript');

// ===================================================================================
// ----------------------------------- Declarations ----------------------------------
// ===================================================================================

Blockly.JavaScript['declaration_nullaryFunction'] = function(block) { 
 
  var sym      = Blockly.JavaScript.valueToCode(block, 'semi_or_epsilon', Blockly.JavaScript.ORDER_ADDITION) || '0';  
  var nameOfFn = block.getFieldValue('nameOfFn');
  
  processBegin('declaration_nullaryFunction','nullary function declaration',sym);
  wellFormedFunName(nameOfFn);
  
  var body     = Blockly.JavaScript.statementToCode(block, 'body', Blockly.JavaScript.ORDER_ADDITION) || '1'; 
  var code     = 'function ' + nameOfFn + ' () ' + '{ ' + body + ' }\n';
    
  processEnd(DEC_END, nameOfFn + ' () ');
  
  
  return code;
};

// -----------------------------------------------------------------------------------
Blockly.JavaScript['declaration_function'] = function(block) { 
 
  var sym      = Blockly.JavaScript.valueToCode(block, 'semi_or_epsilon', Blockly.JavaScript.ORDER_ADDITION) || '0';    
  var nameOfFn = block.getFieldValue('nameOfFn');
  
  processBegin('declaration_function','parameterized function declaration',sym);
  wellFormedFunName(nameOfFn);
  
  var xParam   = block.getFieldValue('x');
  var zParam   = block.getFieldValue('z');
  var body     = Blockly.JavaScript.statementToCode(block, 'body', Blockly.JavaScript.ORDER_ADDITION) || '1'; 
  var code     = 'function ' + nameOfFn + ' (' + xParam + ',' + zParam + ') ' + '{ ' + body + ' }\n';
    
  processEnd(DEC_END, nameOfFn + ' (' + xParam + ',' + zParam + ') ');
  
  
  return code;
};

// ===================================================================================
// ----------------------------------- function calls --------------------------------
// ===================================================================================
  
Blockly.JavaScript['bcallNullary'] = function(block) { 
  
  var nameOfFn = block.getFieldValue('userFunName');
  var sym      = Blockly.JavaScript.valueToCode(block, 'semi_or_epsilon', Blockly.JavaScript.ORDER_ADDITION) || '1'; 
  var code     = nameOfFn + '();\n';
 
  processBegin('call','call',sym); 
  processEnd(CALL,nameOfFn + '()' + sym);
 
  wellFormedFunName(nameOfFn);
 
  return code;
};

  
Blockly.JavaScript['bcall'] = function(block) { 
  
  var nameOfFn = block.getFieldValue('userFunName');
  var xArg     = block.getFieldValue('xArg');
  var zArg     = block.getFieldValue('zArg');
  var sym      = Blockly.JavaScript.valueToCode(block, 'semi_or_epsilon', Blockly.JavaScript.ORDER_ADDITION) || '1'; 
  var code     = nameOfFn + '(' + xArg + ',' + zArg + ');\n';
 
  processBegin('call','call',sym); 
  processEnd(CALL,nameOfFn +  '(' + xArg + ',' + zArg + ')' + sym);
  
  wellFormedFunName(nameOfFn);
  wellFormedInt(xArg);
  wellFormedInt(zArg);
 
  return code;
};
 
