/**
 * @license
 * Visual Blocks Language
 *
 * Copyright 2012 Google Inc.
 * https://developers.google.com/blockly/
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

/**
 * @fileoverview Generating JavaScript for text blocks.
 * @author vwinter@unomaha.edu (Victor Winter)
 */
'use strict';

goog.provide('Blockly.JavaScript.texts');

goog.require('Blockly.JavaScript');

// ===================================================================================
// ------------------------------- Offsets -------------------------------------------
// ===================================================================================
Blockly.JavaScript['setOffset2D'] = function(block) {
    
    var name = 'setOffset2D';
    var x    = block.getFieldValue('xVal');
    var z    = block.getFieldValue('zVal');
    var sym  = Blockly.JavaScript.valueToCode(block, 'semi_or_epsilon', Blockly.JavaScript.ORDER_ADDITION) || '1';     
    var code = 'setOffset(' + x + ',' + z + ');\n';    

    processBegin('offset', name + '(' + x + ',' + z + ')',sym);         
    processEnd  (CALL  , name + '(' + x + ',' + z + ')' + sym);
    
    wellFormedInt(x);
    wellFormedInt(z);
    
    return code;
};

// -----------------------------------------------------------------------------------
Blockly.JavaScript['incOffset2D'] = function(block) {
    
    var name = 'incOffset2D';
    var x    = block.getFieldValue('xVal');
    var z    = block.getFieldValue('zVal');
    var sym  = Blockly.JavaScript.valueToCode(block, 'semi_or_epsilon', Blockly.JavaScript.ORDER_ADDITION) || '1';     
    var code = 'incOffset(' + x + ',' + z + ');\n';    

    processBegin('offset', name + '(' + x + ',' + z + ')',sym);         
    processEnd  (CALL  , name + '(' + x + ',' + z + ')' + sym);

    wellFormedInt(x);
    wellFormedInt(z);
    
    return code;
};

// -----------------------------------------------------------------------------------
Blockly.JavaScript['decOffset2D'] = function(block) {
    
    var name = 'decOffset2D';
    var x    = block.getFieldValue('xVal');
    var z    = block.getFieldValue('zVal');
    var sym  = Blockly.JavaScript.valueToCode(block, 'semi_or_epsilon', Blockly.JavaScript.ORDER_ADDITION) || '1';     
    var code = 'decOffset(' + x + ',' + z + ');\n';    

    processBegin('offset', name + '(' + x + ',' + z + ')',sym);         
    processEnd  (CALL  , name + '(' + x + ',' + z + ')' + sym);

    wellFormedInt(x);
    wellFormedInt(z);
    
    return code;
};


