/**
 * @license
 * Visual Blocks Language
 *
 * Copyright 2012 Google Inc.
 * https://developers.google.com/blockly/
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

/**
 * @fileoverview Generating JavaScript for text blocks.
 * @author vwinter@unomaha.edu (Victor Winter)
 */
'use strict';

goog.provide('Blockly.JavaScript.texts');

goog.require('Blockly.JavaScript');

// ===================================================================================
// ----------------------------------- Shapes ----------------------------------------
// ===================================================================================
Blockly.JavaScript['circleXZ'] = function(block) {
    
    var name   = 'circleXZ';
    var radius = block.getFieldValue('radius');
    var brick  = block.getFieldValue('brick');    
    var x      = block.getFieldValue('xArg');
    var z      = block.getFieldValue('zArg');
    var sym    = Blockly.JavaScript.valueToCode(block, 'semi_or_epsilon', Blockly.JavaScript.ORDER_ADDITION) || '1';     
    var code   = name + '(' + radius + ',' + '\'' + brick + '\'' + ',' + x + ',' + z + ');\n';    

    var brickId = colorToBrick(brick);
    
    processBegin('circleXZ', name + ' ' + radius + ' ' + brickId + ' (' + x + ',' + z + ')',sym);         
    processEnd  (CALL  , name + ' ' + radius + ' ' + brickId + ' (' + x + ',' + z + ')' + sym);

    wellFormedInt(radius);
    
    wellFormedInt(x);
    wellFormedInt(z);
    
    return code;    
}

Blockly.JavaScript['ringXZ'] = function(block) {
    
    var name      = 'ringXZ';
    var radius    = block.getFieldValue('radius');
    var thickness = block.getFieldValue('thickness');
    var brick     = block.getFieldValue('brick');    
    var x         = block.getFieldValue('xArg');
    var z         = block.getFieldValue('zArg');
    var sym       = Blockly.JavaScript.valueToCode(block, 'semi_or_epsilon', Blockly.JavaScript.ORDER_ADDITION) || '1';     
    var code      = name + '(' + radius + ',' + thickness + ',' + '\'' + brick + '\'' + ',' + x + ',' + z + ');\n';    

    var brickId = colorToBrick(brick);
    
    processBegin('ringXZ', name + ' ' + radius + ' ' + thickness + ' ' + brickId +  ' (' + x + ',' + z + ')',sym);         
    processEnd  (CALL    , name + ' ' + radius + ' ' + thickness + ' ' + brickId +  ' (' + x + ',' + z + ')' + sym);
     
    wellFormedInt(radius);
    wellFormedInt(thickness);
    
    wellFormedInt(x);
    wellFormedInt(z);
       
    return code;    
}

